package jdk.internal.util.xml.impl;

import java.nio.charset.UnsupportedCharsetException;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.Charset;
import java.io.UnsupportedEncodingException;
import jdk.internal.util.xml.XMLStreamException;
import java.io.OutputStream;
import jdk.internal.util.xml.XMLStreamWriter;

public class XMLStreamWriterImpl implements XMLStreamWriter
{
    static final int STATE_XML_DECL = 1;
    static final int STATE_PROLOG = 2;
    static final int STATE_DTD_DECL = 3;
    static final int STATE_ELEMENT = 4;
    static final int ELEMENT_STARTTAG_OPEN = 10;
    static final int ELEMENT_STARTTAG_CLOSE = 11;
    static final int ELEMENT_ENDTAG_OPEN = 12;
    static final int ELEMENT_ENDTAG_CLOSE = 13;
    public static final char CLOSE_START_TAG = '>';
    public static final char OPEN_START_TAG = '<';
    public static final String OPEN_END_TAG = "</";
    public static final char CLOSE_END_TAG = '>';
    public static final String START_CDATA = "<![CDATA[";
    public static final String END_CDATA = "]]>";
    public static final String CLOSE_EMPTY_ELEMENT = "/>";
    public static final String ENCODING_PREFIX = "&#x";
    public static final char SPACE = ' ';
    public static final char AMPERSAND = '&';
    public static final char DOUBLEQUOT = '\"';
    public static final char SEMICOLON = ';';
    private int _state;
    private XMLStreamWriterImpl.Element _currentEle;
    private XMLWriter _writer;
    private String _encoding;
    boolean _escapeCharacters;
    private boolean _doIndent;
    private char[] _lineSep;
    
    public XMLStreamWriterImpl(OutputStream outputStream) throws XMLStreamException {
        this(outputStream, "UTF-8");
    }
    
    public XMLStreamWriterImpl(OutputStream outputStream, String encoding) throws XMLStreamException {
        this._state = 0;
        this._escapeCharacters = true;
        this._doIndent = true;
        this._lineSep = System.getProperty("line.separator").toCharArray();
        Charset charset = null;
        if (encoding == null) {
            this._encoding = "UTF-8";
        }
        else {
            try {
                charset = this.getCharset(encoding);
            }
            catch (UnsupportedEncodingException th) {
                throw new XMLStreamException(th);
            }
            this._encoding = encoding;
        }
        this._writer = new XMLWriter(outputStream, encoding, charset);
    }
    
    @Override
    public void writeStartDocument() throws XMLStreamException {
        this.writeStartDocument(this._encoding, "1.0");
    }
    
    @Override
    public void writeStartDocument(String s) throws XMLStreamException {
        this.writeStartDocument(this._encoding, s, null);
    }
    
    @Override
    public void writeStartDocument(String s, String s2) throws XMLStreamException {
        this.writeStartDocument(s, s2, null);
    }
    
    public void writeStartDocument(String encoding, String s, String s2) throws XMLStreamException {
        if (this._state > 0) {
            throw new XMLStreamException("XML declaration must be as the first line in the XML document.");
        }
        this._state = 1;
        if ((encoding = encoding) == null) {
            encoding = this._encoding;
        }
        else {
            try {
                this.getCharset(encoding);
            }
            catch (UnsupportedEncodingException th) {
                throw new XMLStreamException(th);
            }
        }
        if (s == null) {
            s = "1.0";
        }
        this._writer.write("<?xml version=\"");
        this._writer.write(s);
        this._writer.write(34);
        if (encoding != null) {
            this._writer.write(" encoding=\"");
            this._writer.write(encoding);
            this._writer.write(34);
        }
        if (s2 != null) {
            this._writer.write(" standalone=\"");
            this._writer.write(s2);
            this._writer.write(34);
        }
        this._writer.write("?>");
        this.writeLineSeparator();
        return;
    }
    
    @Override
    public void writeDTD(String s) throws XMLStreamException {
        if (this._currentEle != null && this._currentEle.getState() == 10) {
            this.closeStartTag();
        }
        this._writer.write(s);
        this.writeLineSeparator();
    }
    
    @Override
    public void writeStartElement(String s) throws XMLStreamException {
        if (s == null || s.length() == 0) {
            throw new XMLStreamException("Local Name cannot be null or empty");
        }
        this._state = 4;
        if (this._currentEle != null && this._currentEle.getState() == 10) {
            this.closeStartTag();
        }
        this._currentEle = new XMLStreamWriterImpl.Element(this._currentEle, s, false);
        this.openStartTag();
        this._writer.write(s);
        return;
    }
    
    @Override
    public void writeEmptyElement(String s) throws XMLStreamException {
        if (this._currentEle != null && this._currentEle.getState() == 10) {
            this.closeStartTag();
        }
        this._currentEle = new XMLStreamWriterImpl.Element(this._currentEle, s, true);
        this.openStartTag();
        this._writer.write(s);
    }
    
    @Override
    public void writeAttribute(String s, String s2) throws XMLStreamException {
        if (this._currentEle.getState() != 10) {
            throw new XMLStreamException("Attribute not associated with any element");
        }
        this._writer.write(32);
        this._writer.write(s);
        this._writer.write("=\"");
        this.writeXMLContent(s2, true, true);
        this._writer.write(34);
        return;
    }
    
    @Override
    public void writeEndDocument() throws XMLStreamException {
        if (this._currentEle != null && this._currentEle.getState() == 10) {
            this.closeStartTag();
        }
        while (this._currentEle != null) {
            if (!this._currentEle.isEmpty()) {
                this._writer.write("</");
                this._writer.write(this._currentEle.getLocalName());
                this._writer.write(62);
            }
            this._currentEle = this._currentEle.getParent();
        }
    }
    
    @Override
    public void writeEndElement() throws XMLStreamException {
        if (this._currentEle != null && this._currentEle.getState() == 10) {
            this.closeStartTag();
        }
        if (this._currentEle == null) {
            throw new XMLStreamException("No element was found to write");
        }
        if (this._currentEle.isEmpty()) {
            return;
        }
        this._writer.write("</");
        this._writer.write(this._currentEle.getLocalName());
        this._writer.write(62);
        this.writeLineSeparator();
        this._currentEle = this._currentEle.getParent();
        return;
    }
    
    @Override
    public void writeCData(String s) throws XMLStreamException {
        if (s == null) {
            throw new XMLStreamException("cdata cannot be null");
        }
        if (this._currentEle != null && this._currentEle.getState() == 10) {
            this.closeStartTag();
        }
        this._writer.write("<![CDATA[");
        this._writer.write(s);
        this._writer.write("]]>");
        return;
    }
    
    @Override
    public void writeCharacters(String s) throws XMLStreamException {
        if (this._currentEle != null && this._currentEle.getState() == 10) {
            this.closeStartTag();
        }
        this.writeXMLContent(s);
    }
    
    @Override
    public void writeCharacters(char[] array, int n, int n2) throws XMLStreamException {
        if (this._currentEle != null && this._currentEle.getState() == 10) {
            this.closeStartTag();
        }
        this.writeXMLContent(array, n, n2, this._escapeCharacters);
    }
    
    @Override
    public void close() throws XMLStreamException {
        if (this._writer != null) {
            this._writer.close();
        }
        this._writer = null;
        this._currentEle = null;
        this._state = 0;
    }
    
    @Override
    public void flush() throws XMLStreamException {
        if (this._writer != null) {
            this._writer.flush();
        }
    }
    
    public void setDoIndent(boolean doIndent) {
        this._doIndent = doIndent;
    }
    
    private void writeXMLContent(char[] array, int i, int n, boolean b) throws XMLStreamException {
        if (!b) {
            this._writer.write(array, i, n);
            return;
        }
        int n2;
        int j;
        for (, n2 = i + n, ; i < n2; ++i) {
            j = array[i];
            if (!this._writer.canEncode((char)j)) {
                this._writer.write(array, i, i - i);
                this._writer.write("&#x");
                this._writer.write(Integer.toHexString(j));
                this._writer.write(59);
                ++i;
            }
            else {
                switch (j) {
                    case '<': {
                        this._writer.write(array, i, i - i);
                        this._writer.write("&lt;");
                        ++i;
                        break;
                    }
                    case '&': {
                        this._writer.write(array, i, i - i);
                        this._writer.write("&amp;");
                        ++i;
                        break;
                    }
                    case '>': {
                        this._writer.write(array, i, i - i);
                        this._writer.write("&gt;");
                        ++i;
                        break;
                    }
                }
            }
        }
        this._writer.write(array, i, n2 - i);
        return;
    }
    
    private void writeXMLContent(String s) throws XMLStreamException {
        if (s != null && s.length() > 0) {
            this.writeXMLContent(s, this._escapeCharacters, false);
        }
    }
    
    private void writeXMLContent(String s, boolean b, boolean b2) throws XMLStreamException {
        if (!b) {
            this._writer.write(s);
            return;
        }
        int n = 0;
        int length = s.length();
        for (int i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            if (!this._writer.canEncode((char)char1)) {
                this._writer.write(s, n, i - n);
                this._writer.write("&#x");
                this._writer.write(Integer.toHexString(char1));
                this._writer.write(59);
                n = i + 1;
            }
            else {
                switch (char1) {
                    case '<': {
                        this._writer.write(s, n, i - n);
                        this._writer.write("&lt;");
                        n = i + 1;
                        break;
                    }
                    case '&': {
                        this._writer.write(s, n, i - n);
                        this._writer.write("&amp;");
                        n = i + 1;
                        break;
                    }
                    case '>': {
                        this._writer.write(s, n, i - n);
                        this._writer.write("&gt;");
                        n = i + 1;
                        break;
                    }
                    case '\"': {
                        this._writer.write(s, n, i - n);
                        if (b2) {
                            this._writer.write("&quot;");
                        }
                        else {
                            this._writer.write(34);
                        }
                        n = i + 1;
                        break;
                    }
                }
            }
        }
        this._writer.write(s, n, length - n);
        return;
    }
    
    private void openStartTag() throws XMLStreamException {
        this._currentEle.setState(10);
        this._writer.write(60);
    }
    
    private void closeStartTag() throws XMLStreamException {
        if (this._currentEle.isEmpty()) {
            this._writer.write("/>");
        }
        else {
            this._writer.write(62);
        }
        if (this._currentEle.getParent() == null) {
            this.writeLineSeparator();
        }
        this._currentEle.setState(11);
    }
    
    private void writeLineSeparator() throws XMLStreamException {
        if (this._doIndent) {
            this._writer.write(this._lineSep, 0, this._lineSep.length);
        }
    }
    
    private Charset getCharset(String str) throws UnsupportedEncodingException {
        if (str.equalsIgnoreCase("UTF-32")) {
            throw new UnsupportedEncodingException("The basic XMLWriter does not support " + str);
        }
        Charset forName;
        try {
            forName = Charset.forName(str);
        }
        catch (IllegalCharsetNameException | UnsupportedCharsetException ex) {
            throw new UnsupportedEncodingException(str);
        }
        return forName;
    }
    
    protected class Element
    {
        protected XMLStreamWriterImpl.Element _parent;
        protected short _Depth;
        boolean _isEmptyElement;
        String _localpart;
        int _state;
        
        public Element() {
            this._isEmptyElement = false;
        }
        
        public Element(XMLStreamWriterImpl.Element parent, String localpart, boolean isEmptyElement) {
            this._isEmptyElement = false;
            this._parent = parent;
            this._localpart = localpart;
            this._isEmptyElement = isEmptyElement;
        }
        
        public XMLStreamWriterImpl.Element getParent() {
            return this._parent;
        }
        
        public String getLocalName() {
            return this._localpart;
        }
        
        public int getState() {
            return this._state;
        }
        
        public void setState(int state) {
            this._state = state;
        }
        
        public boolean isEmpty() {
            return this._isEmptyElement;
        }
    }
}
